"""Config introspection support."""


__all__ = ("ConfigHint", "configurable")


class ConfigHint:
    """Hint for introspection supplying overrides."""

    # be aware this is used in clone
    __slots__ = (
        "types",
        "positional",
        "required",
        "typename",
        "allow_unknowns",
        "doc",
        "authorative",
        "requires_config",
        "raw_class",
    )

    def __init__(
        self,
        types=None,
        positional=None,
        required=None,
        doc=None,
        typename=None,
        allow_unknowns=False,
        authorative=False,
        requires_config=False,
        raw_class=False,
    ):
        self.types = types or {}
        self.positional = positional or []
        self.required = required or []
        self.typename = typename
        self.allow_unknowns = allow_unknowns
        self.doc = doc
        self.authorative = authorative
        self.requires_config = requires_config
        self.raw_class = raw_class

    def clone(self, **kwds):
        new_kwds = {}
        for attr in self.__slots__:
            new_kwds[attr] = kwds.pop(attr, getattr(self, attr))
        if kwds:
            raise TypeError("unknown type overrides: %r" % kwds)
        return self.__class__(**new_kwds)


def configurable(*args, **kwargs):
    """Decorator version of ConfigHint."""
    hint = ConfigHint(*args, **kwargs)

    def decorator(original):
        original.pkgcore_config_type = hint
        return original

    return decorator
